import biisan
import os
import codecs
from collections import OrderedDict
from glob import glob
import logging
from multiprocessing import Pool
from email.utils import formatdate
from datetime import datetime
import xml.etree.ElementTree as ET

from css_html_js_minify import html_minify
from docutils.core import publish_parts
from docutils.parsers.rst import directives
from glueplate import config
from jinja2 import Environment, FileSystemLoader

from biisan.utils import get_klass, get_function
from biisan.processors import FunctionRegistry

logging.basicConfig(level=config.settings.log_level)
logger = logging.getLogger(__name__)
processor_registry = None


def unmarshal_story(pth):
    story_class = get_klass(config.settings.story_class)
    with codecs.open(pth, encoding='utf8') as f:
        logger.debug('Unmarshal : {0}'.format(pth))
        data = f.read()
        parts = publish_parts(data, writer_name='xml')
        document = ET.fromstring(parts.get('whole'))
        _story = story_class()
        _story.rst_file = pth
        processor_registry.process(document, _story)
        return _story


def extract_year_month(story_list):
    result = OrderedDict()
    for story in story_list:
        _months = result.get(story.date.year, [])
        _months.append(story.date.month)
        result[story.date.year] = _months
    for key in result.keys():
        result[key] = sorted(list(set(result.get(key))))
    return result


def pack_story_to_year_month(story_list):
    result = OrderedDict()
    for story in story_list:
        _year_month = '{0:04d}/{1:02d}'.format(
            story.date.year, story.date.month)
        _stories = result.get(_year_month, [])
        _stories.append(story)
        result[_year_month] = _stories
    return result


def glob_rst_documents(base_path):
    pool = Pool(config.settings.multiprocess)
    story_list = pool.map(
        unmarshal_story,
        list(glob('{0}/**/*.rst'.format(base_path), recursive=True))
    )
    pool.close()
    pool.join()
    story_list.sort()
    return story_list


def write_html(story):
    os.makedirs(story.directory, exist_ok=True)
    _file = os.path.join(story.directory, 'index.html')
    _data = html_minify(story.to_html())
    _current = None
    if os.path.exists(_file):
        with codecs.open(_file, 'r', 'utf8') as f:
            _current = f.read()
    if _current == _data:
        return
    with codecs.open(_file, 'w', 'utf8') as f:
        f.write(_data)
        logger.info('Write:{0}'.format(_file))


def output(story_list):
    for i, story in enumerate(story_list):
        story.prepare_html(story_list, i)
        write_html(story)


def write_extra(extra):
    extra_page = unmarshal_story('./extra/{0}.rst'.format(extra))
    extra_page.extra = extra
    extra_page.extra_directory(extra)
    output([extra_page])
    return extra_page


def write_top(context):
    env = Environment(loader=FileSystemLoader(config.settings.template_dirs))
    top = env.get_template('index.html')
    with codecs.open(
        os.path.join(
            config.settings.dir.output, 'index.html'), 'w', 'utf8') as f:
        f.write(top.render(**context))


def write_blog_top(story_list):
    cnt = config.settings.latest_list_count * -1 - 1
    latest_story_list = story_list[:cnt:-1]
    year_month = extract_year_month(story_list)
    env = Environment(loader=FileSystemLoader(config.settings.template_dirs))
    blog_top = env.get_template('blog_top.html')
    with codecs.open(
        os.path.join(
            config.settings.dir.output, 'blog', 'index.html'),
            'w', 'utf8') as f:
        f.write(blog_top.render(config=config,
                latest_story_list=latest_story_list,
                story_list=story_list, year_month=year_month))


def write_blog_archive(story_list):
    packed = pack_story_to_year_month(story_list)
    env = Environment(loader=FileSystemLoader(config.settings.template_dirs))
    blog_archive = env.get_template('blog_archive.html')
    for _year_month, stories in packed.items():
        with codecs.open(
            os.path.join(
                config.settings.dir.output, 'blog', _year_month,
                'index.html'),
                'w', 'utf8') as f:
            f.write(blog_archive.render(config=config,
                    year_month=_year_month, story_list=stories))


def write_rss20(story_list):
    now_rfc2822 = formatdate(float(datetime.now().strftime('%s')))
    cnt = config.settings.latest_list_count * -1 - 1
    latest_story_list = story_list[:cnt:-1]
    env = Environment(loader=FileSystemLoader(config.settings.template_dirs))
    rss20 = env.get_template('rss20.xml')
    rss = rss20.render(config=config,
                       story_list=latest_story_list,
                       now_rfc2822=now_rfc2822)
    feed_dir = os.path.join(config.settings.dir.output, 'api', 'feed')
    os.makedirs(feed_dir, exist_ok=True)
    with codecs.open(os.path.join(feed_dir, 'index.xml'), 'w', 'utf8') as f:
        f.write(rss)


def write_sitemaps(story_list):
    last_modified_iso_8601 = max(map(lambda x: x.date, story_list)).isoformat()
    env = Environment(loader=FileSystemLoader(config.settings.template_dirs))
    sitemaps = env.get_template('sitemaps.xml')
    sitemap = sitemaps.render(config=config,
                              story_list=story_list,
                              last_modified=last_modified_iso_8601)
    sitemap_dir = os.path.join(
        config.settings.dir.output, 'api', 'google_sitemaps')
    os.makedirs(sitemap_dir, exist_ok=True)
    with codecs.open(os.path.join(sitemap_dir, 'index.xml'), 'w', 'utf8') as f:
        f.write(sitemap)


def register_directives():
    for directive in config.settings.directives:
        directive_class = get_klass(directive)
        directives.register_directive(
            directive_class.directive_tag,
            directive_class
        )
        logger.debug(directive_class)


def register_processor():
    global processor_registry
    processor_registry = FunctionRegistry()
    for processor in config.settings.processors:
        func = get_function(processor)
        processor_registry.register(func.__name__, func)


def print_fire_message():
    m = '''BIISAN {0}'''.format(biisan.__version__)
    print(m)


def prepare():
    register_directives()
    register_processor()


def main():
    story_list = glob_rst_documents('./blog')
    if len(story_list) == 0:
        logger.error('NO ENTRY FOUND.')
        return
    output(story_list)
    context = {}
    context['config'] = config
    context['story_list'] = story_list
    for extra in config.settings.extra:
        context[extra] = write_extra(extra)
    write_top(context)
    write_blog_top(story_list)
    write_blog_archive(story_list)
    write_rss20(story_list)
    write_sitemaps(story_list)


if __name__ == '__main__':
    print_fire_message()
    prepare()
    main()



from datetime import datetime
import multiprocessing
import os
import sys

from biisan.constants import (
    ABOUT_TMPL,
    BIISAN_DATA_DIR,
    QUESTIONS,
    SETTINGS_TMPL,
)

from PyInquirer import prompt


def check_already_init(data_dir):
    if os.path.exists(data_dir):
        sys.exit('biisan data directory, {0} exists already.'.format(BIISAN_DATA_DIR))


def initialize_structures(data_dir, answer):
    check_already_init(data_dir)
    os.mkdir(data_dir)
    os.mkdir(os.path.join(data_dir, 'data'))
    os.mkdir(os.path.join(data_dir, 'data', 'blog'))
    os.mkdir(os.path.join(data_dir, 'data', 'templates'))
    os.mkdir(os.path.join(data_dir, 'data', 'extra'))
    os.mkdir(os.path.join(data_dir, 'out'))
    with open(os.path.join(data_dir, 'data', 'biisan_local_settings.py'), 'w') as f:
        f.write(
            SETTINGS_TMPL.format(
                multicore=multiprocessing.cpu_count(),
                **answer,
            ),
        )
    n = datetime.now()
    with open(os.path.join(data_dir, 'data', 'extra', 'about.rst'), 'w') as f:
        f.write(
            ABOUT_TMPL.format(
                year=n.year, month=n.month, day=n.day,
                hour=n.hour, minute=n.minute,
                **answer,
            ),
        )


def init():
    data_dir = os.path.join(os.getcwd(), BIISAN_DATA_DIR)
    check_already_init(data_dir)
    answer = prompt(QUESTIONS)
    initialize_structures(data_dir, answer)
    print('''
        Always set environment variable BIISAN_SETTINGS_MODULE to biisan_local_settings like bellow.

        $ export BIISAN_SETTINGS_MODULE=biisan_local_settings
        ''')


if __name__ == '__main__':
    init()



import os
import multiprocessing

version_info = (0, 8, 2)
__version__ = ".".join([str(v) for v in version_info])

os.environ['GLUE_PLATE_BASE_MODULE'] = 'biisan.biisan_settings'

multiprocessing.set_start_method('fork')



from importlib import import_module


def get_klass(pth):
    sep_index = pth.rfind('.')
    module_name = pth[:sep_index]
    class_name = pth[sep_index + 1:]
    mod = import_module(module_name)
    return getattr(mod, class_name)


get_function = get_klass



import os
from email.utils import formatdate
import logging

from glueplate import config
from jinja2 import Environment, FileSystemLoader


logger = logging.getLogger(__name__)


class Container(object):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__body = []

    def __append_to_body(self, content):
        pass

    def add_content(self, content):
        if issubclass(self.__class__, Nestable) and (type(self) == type(content)):
            content.depth = self.depth + 1
        self.__append_to_body(content)
        self.__body.append(content)

    @property
    def contents(self):
        return self.__body


class Nestable(object):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.depth = kwargs.get('depth', 1)


class HTMLize(object):
    env = Environment(loader=FileSystemLoader(config.settings.template_dirs))

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def to_html(self):
        tmpl = HTMLize.env.get_template(
            os.path.join(
                'components',
                '{0}.html'.format(self.__class__.__name__).lower()
            )
        )
        return tmpl.render(element=self, config=config)


class Story(Container, HTMLize):
    def __init__(self):
        super().__init__()
        self.slug = ''
        self.title = ''
        self.__date = None
        self.author = ''
        self.__body = []
        self.comments = []
        self._timestamp = None
        self.rst_file = ''
        self.extra = None

    def __lt__(self, other):
        try:
            return self._timestamp <= other._timestamp
        except TypeError as e:
            logger.error('-' * 20)
            logger.error(self.rst_file)
            logger.error(self.slug)
            logger.error(self.__body)
            logger.error('=' * 20)
            raise e

    def __repr__(self):
        return '{0}: {1} at {2}, {3} comments'.format(
            self.slug, self.title, self.__date, len(self.comments))

    @property
    def date(self):
        if self.__date is None:
            raise ValueError('date must not be None.')
        return self.__date

    @date.setter
    def date(self, date):
        self.__date = date
        self._timestamp = self.__date.timestamp()

    @property
    def directory(self):
        if not hasattr(self, '_directory'):
            self._directory = os.path.join(
                '{0}'.format(config.settings.dir.output),
                'blog',
                '{0:04d}'.format(self.__date.year),
                '{0:02d}'.format(self.__date.month),
                '{0:02d}'.format(self.__date.day),
                self.slug
            )
        return self._directory

    @property
    def archive_directory(self):
        return os.path.join(
            '{0}'.format(config.settings.dir.output),
            'archive',
            '{0:04d}'.format(self.__date.year),
            '{0}'.format(self.__date.month)
        )

    @property
    def url(self):
        return '/blog/{0:04d}/{1:02d}/{2:02d}/{3}/{4}'.format(
            self.__date.year,
            self.__date.month,
            self.__date.day,
            self.slug,
            '')

    @property
    def publishd_date(self):
        return '{0:04d}-{1:02d}-{2:02d}/'.format(
            self.__date.year,
            self.__date.month,
            self.__date.day)

    @property
    def published_datetime(self):
        return '{0:04d}/{1:02d}/{2:02d} {3:02d}:{4:02d}'.format(
            self.__date.year,
            self.__date.month,
            self.__date.day,
            self.__date.hour,
            self.__date.minute)

    @property
    def publish_date_rfc2822(self):
        return formatdate(float(self.__date.strftime('%s')))

    def prepare_html(self, story_list, i):
        self.prev_story = previous_story(story_list, i)
        self.next_story = next_story(story_list, i)

    def extra_directory(self, directory):
        self._directory = os.path.join(
            '{0}'.format(config.settings.dir.output),
            directory)


def archive_directory(year_month):
    return os.path.join(
        '{0}'.format(config.settings.dir.output),
        'archive', year_month)


class Comment(Container):
    def __init__(self):
        super().__init__()
        self.commentator = ''
        self.url = ''
        self.create_date = None

    # def add_content(self, content):
    #     self.__body.append(content)

    # @property
    # def contents(self):
    #     return self.__body

    @property
    def comemnted_datetime(self):
        return '{0:04d}/{1:02d}/{2:02d} {3:02d}:{4:02d}'.format(
            self.create_date.year,
            self.create_date.month,
            self.create_date.day,
            self.create_date.hour,
            self.create_date.minute)


def next_story(story_list, i):
    if i >= len(story_list) - 1:
        return '', ''
    target = story_list[i + 1]
    return target.title, target.url


def previous_story(story_list, i):
    if i == 0:
        return '', ''
    target = story_list[i - 1]
    return target.title, target.url


class Document():
    pass


class Paragraph(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = ''

    @property
    def formated(self):
        _formated = self.text
        for content in self.contents:
            if isinstance(content, Strong):
                _formated = _formated.replace(
                    content.text, '<strong>{0}</strong>'.format(
                        content.text))
            elif isinstance(content, Emphasis):
                _formated = _formated.replace(
                    content.text, '<i>{0}</i>'.format(
                        content.text))
            elif isinstance(content, Reference):
                _name = content.name and content.name or content.uri
                _formated = _formated.replace(
                    content.text,
                    '<a href="{0}">{1}</a>'.format(
                        content.uri, _name))
            elif isinstance(content, Raw):
                _formated = _formated.replace(
                    content.text,
                    '<pre class="code {0}">{0}</pre>'.format(
                        content.format, content.text))
            else:
                logger.warning(
                    "Type:{0} in paragraph doesn't treat.".format(
                        type(content)))
        return _formated


class Strong(Document, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = ''


class Emphasis(Document, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = ''


class Section(Document, Container, Nestable, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.title = ''


class BulletList(Document, Container, Nestable, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class EnumeratedList(Document, Container, Nestable, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ListItem(Document, Container, Nestable, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Title(Document):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = ''

    def __repr__(self):
        return self.text or ''


class Target(Document, HTMLize):
    def __init__(self):
        super().__init__()
        self.ids = ''
        self.names = ''
        self.uri = ''


class Reference(Document, HTMLize):
    def __init__(self):
        super().__init__()
        self.name = ''
        self.uri = ''


class Raw(Document, HTMLize):
    def __init__(self):
        super().__init__()
        self.format = ''
        self.text = ''


class Image(Document, HTMLize):
    def __init__(self):
        super().__init__()
        self.alt = ''
        self.uri = ''
        self._width = None
        self._height = None

    @property
    def width(self):
        if not self._width:
            return ''
        return self._width

    @width.setter
    def width(self, value):
        self._width = value

    @property
    def height(self):
        if not self._height:
            return ''
        return self._height

    @height.setter
    def height(self, value):
        self._height = value


class BlockQuote(Document, Container, Nestable, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class LiteralBlock(Document, Container, Nestable, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = kwargs.get('text', '')


class Figure(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Caption(Document, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = ''


class Table(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.title = Title()


class Thead(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Tbody(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Tgroup(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ColSpec(Document, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.colname = ''
        self.width = None
        self.scale = 100


class Row(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.header = False

    def to_html(self):
        if not self.header:
            return super().to_html()
        tmpl = HTMLize.env.get_template(
            os.path.join(
                'components',
                'header_{0}.html'.format(self.__class__.__name__).lower()
            )
        )
        return tmpl.render(element=self, config=config)


class Entry(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Transition(Document, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Topic(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.title = Title()


class SubstitutionDefinition(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.title = Title()


class Note(Document, Container, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class DefinitionList(Document, Container, Nestable, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class Term(Document):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = kwargs.get('text', '')


class Definition(Document, Container):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class DefinitionListItem(Document, HTMLize):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.term = Term()
        self.definition = Definition()



from glueplate import Glue as _
import logging
import os


settings = _(
    GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY='BIISAN_SETTINGS_MODULE',
    story_class='biisan.models.Story',
    latest_list_count=5,
    processors=[
        'biisan.processors.process_document',
        'biisan.processors.process_title',
        'biisan.processors.process_docinfo',
        'biisan.processors.process_field_name',
        'biisan.processors.process_field_body',
        'biisan.processors.process_paragraph',
        'biisan.processors.process_section',
        'biisan.processors.process_bullet_list',
        'biisan.processors.process_list_item',
        'biisan.processors.process_target',
        'biisan.processors.process_raw',
        'biisan.processors.process_image',
        'biisan.processors.process_block_quote',
        'biisan.processors.process_literal_block',
        'biisan.processors.process_figure',
        'biisan.processors.process_caption',
        'biisan.processors.process_table',
        'biisan.processors.process_tgroup',
        'biisan.processors.process_colspec',
        'biisan.processors.process_thead',
        'biisan.processors.process_row',
        'biisan.processors.process_entry',
        'biisan.processors.process_tbody',
        'biisan.processors.process_enumerated_list',
        'biisan.processors.process_transition',
        'biisan.processors.process_topic',
        'biisan.processors.process_substitution_definition',
        'biisan.processors.process_note',
        'biisan.processors.process_definition_list',
        'biisan.processors.process_definition_list_item',
        'biisan.processors.process_definition',
        'biisan.processors.process_strong',
        'biisan.processors.process_emphasis',
        'biisan.processors.process_reference',
    ],
    directives=[
        'biisan.directives.PrismDirective',
        'biisan.directives.NotesDirective',
        'biisan.directives.AffDirective',
        'biisan.directives.AppleAffButtonDirective'
    ],
    directive=_(
        aff=_(
            tld='co.jp',
            tag='everes-22',
        ),
    ),
    extra=[
        'about',
    ],
    template_dirs=[os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        'templates'), ],
    multiprocess=4,
    log_level=logging.INFO,
    dir=_(
        output='~/Desktop/biisan'
    )
)



SETTINGS_TMPL = '''import os
from glueplate import Glue as _

settings = _(
    blog = _(
        title = '{blog_title}',
        base_url = '{base_url}',
        language = '{language}',
    ),
    dir = _(
        output = os.path.abspath(os.path.join('..', 'out'))
    ),
    GLUE_PLATE_PLUS_BEFORE_template_dirs = [os.path.abspath(os.path.join('.', 'templates')),],
    multiprocess = {multicore},
)
'''

ABOUT_TMPL = '''About {blog_title}
=========================================================

:slug: about
:date: {year}-{month:02d}-{day:02d} {hour:02d}:{minute:02d}

'''

QUESTIONS = [
    {
        'type': 'input',
        'name': 'blog_title',
        'message': 'What\'s your blog title',
    },
    {
        'type': 'input',
        'name': 'base_url',
        'message': 'input your blog base url. like https://www.tsuyukimakoto.com',
    },
    {
        'type': 'input',
        'name': 'language',
        'message': 'input your blog language like ja',
    },
]

BIISAN_DATA_DIR = 'biisan_data'



from html import escape
import logging

from docutils import nodes
from docutils.parsers.rst import directives, Directive

from glueplate import config

logger = logging.getLogger(__name__)


class PrismDirective(Directive):
    """
prism::
  :language: bash
    """

    directive_tag = 'prism'
    has_content = True
    option_spec = {'language': directives.unchanged}

    def run(self):
        css_class = ''
        if 'language' in self.options:
            css_class = 'language-{0}'.format(self.options['language'])
        self.assert_has_content()
        text = '<pre><code class="{0}">{1}</code></pre>'.format(
            css_class, self._get_escaped_content())
        return [nodes.raw('', text, format='html')]

    def _get_escaped_content(self):
        return '\n'.join(map(escape, self.content))


class NotesDirective(Directive):
    """
    """

    directive_tag = 'notes'
    has_content = True
    option_spec = {'date': directives.unchanged}

    def run(self):
        date_str = ''
        if 'date' in self.options:
            date_str = '({0})'.format(self.options['date'])
        self.assert_has_content()
        text = '''
<div class="notes">
  <blockquote class="last"><i class="icon-info"></i>{0} {1}</blockquote>
</div>'''.format(self._get_escaped_content(), date_str)
        return [nodes.raw('', text, format='html')]

    def _get_escaped_content(self):
        return '\n'.join(map(escape, self.content))


class AffDirective(Directive):
    """
aff::
  :asin: ASIN
  :title: title
  :aftag: tag
    """

    directive_tag = 'aff'
    has_content = True
    option_spec = {'asin': directives.unchanged, 'title': directives.unchanged}

    def run(self):
        _asin = '{0}'.format(self.options['asin'])
        _title = '{0}'.format(self.options['title'])
        self.assert_has_content()
        text = '''
<div class="biisan-aff">
  <fieldset>
    <legend>{title}</legend>
    <div class="biisan-aff-container">
      <div class="biisan-aff_amz">
        <img src="http://images-jp.amazon.com/images/P/{asin}.09.SZZZZZZZ.jpg">
        <a href="http://www.amazon.{tld}/gp/product/{asin}?tag={tag}">
          <h4>{title}</h4>
          <img
           src="http://ecx.images-amazon.com/images/G/09/buttons/buy-from-tan.gif" />
        </a>
      </div>
      <div class="biisan-aff_content">
      {contents}
      </div>
    </div>
  </fieldset>
</div>'''.format(asin=_asin, title=_title,
                 tld=config.settings.directive.aff.tld,
                 tag=config.settings.directive.aff.tag,
                 contents='<br />'.join(self.content))
        return [nodes.raw('', text, format='html')]


class AppleAffButtonDirective(Directive):
    """
appleaff::
  :url: permalink
  :shop: appstore / macappstore / itunes / music
    """

    directive_tag = 'appleaff'
    has_content = False
    option_spec = {'at': directives.unchanged,
                   'url': directives.unchanged,
                   'shop': directives.unchanged}
    shop_type_button = {
        'appstore': 'https://linkmaker.itunes.apple.com/images/badges/ja-jp/badge_appstore-lrg.svg',
        'macappstore': 'https://linkmaker.itunes.apple.com/images/badges/ja-jp/badge_macappstore-lrg.svg',
        'itunes': 'https://linkmaker.itunes.apple.com/images/badges/ja-jp/badge_itunes-lrg.svg',
        'music': 'https://linkmaker.itunes.apple.com/images/badges/ja-jp/badge_music-lrg.svg'
    }

    def get_shop(self, shop_type):
        assert(shop_type in AppleAffButtonDirective.shop_type_button)
        return AppleAffButtonDirective.shop_type_button[shop_type]

    def run(self):
        _url = '{0}'.format(self.options['url'])
        _shop = '{0}'.format(self.options['shop'])
        _shop_button = self.get_shop(_shop)
        _affurl = '?' in _url and '{0}&at={1}' or '{0}?at={1}'
        _affurl = _affurl.format(_url, config.settings.directive.appleaff.at)
        text = '''<a class="biisan-apple-aff" href="{affurl}"><img
  src="{shop_button}" /></a>'''.format(affurl=_affurl, shop_button=_shop_button)
        return [nodes.raw('', text, format='html')]



#!/usr/bin/env python
'''
Biisan
============================================================

Static blog site generator.
'''
from setuptools import setup

from biisan import __version__

requirements = []
with open('requirements/requirements.txt', 'r') as f:
    requirements = [req.strip() for req in f.readlines() if req]

with open('README.md') as f:
    long_description = f.read()

setup(
    name='biisan',
    version=__version__,
    description='Static site generator.',
    keywords='static site generator, docutils, customizable',
    author='makoto tsuyuki',
    author_email='mtsuyuki@gmail.com',
    url='https://github.com/tsuyukimakoto/biisan',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Documentation',
    ],
    zip_safe=False,
    install_requires=requirements,
    packages=['biisan', 'biisan.directives', 'biisan.processors'],
    package_data={
          'biisan': ['templates/*', 'templates/components/*', ],
    },
)



from datetime import datetime
import logging

from biisan.models import (
    Comment, Paragraph, Section, BulletList, ListItem, Target, Raw, Image, BlockQuote, Title,
    LiteralBlock, Figure, Caption, Table, Thead, Tbody, Tgroup, ColSpec, Row, Entry, EnumeratedList, Transition,
    Topic, SubstitutionDefinition, Note, DefinitionList, DefinitionListItem, Term, Definition,
    Strong, Emphasis, Reference
)

logger = logging.getLogger(__name__)


def _debug(elm):
    logger.debug('-' * 20)
    logger.debug('Tag: {0}'.format(elm.tag))
    logger.debug('text: {0}'.format(elm.text))
    logger.debug('------- items --------------')
    logger.debug(elm.items())
    logger.debug('------- getchildren --------')
    logger.debug(list(elm))
    logger.debug('-' * 20)


def process_field_name(elm, registry, container):
    return elm.text


def process_field_body(elm, registry, container):
    res = []
    for x in list(elm):
        if 'field_list' == x.tag:
            logger.warning("Ignore field_list in field_body's child")
        else:
            res.append(x.text)
    return res


def process_list_item(elm, registry, container):
    list_item = ListItem()
    container.add_content(list_item)
    for _elm in list(elm):
        registry.process(_elm, list_item)


def process_bullet_list(elm, registry, container):
    bullet_list = BulletList()
    container.add_content(bullet_list)
    for _elm in list(elm):
        registry.process(_elm, bullet_list)


def process_definition_list(elm, registry, container):
    definition_list = DefinitionList()
    container.add_content(definition_list)
    for _elm in list(elm):
        registry.process(_elm, definition_list)


def process_definition(elm, registry, container):
    definition = Definition()
    container.definition = definition
    for _elm in list(elm):
        registry.process(_elm, definition)


def process_definition_list_item(elm, registry, container):
    definition_list_item = DefinitionListItem()
    container.add_content(definition_list_item)
    for _elm in list(elm):
        if _elm.tag == 'term':
            term = Term()
            term.text = _elm.text
            definition_list_item.term = term
        elif _elm.tag == 'definition':
            registry.process(_elm, definition_list_item)


def process_target(elm, registry, container):
    target = Target()
    container.add_content(target)
    for subitem in elm.items():
        if subitem[0] == 'ids':
            target.ids = subitem[1]
        elif subitem[0] == 'names':
            target.names = subitem[1]
        elif subitem[0] == 'uri' or subitem[0] == 'refuri':
            target.uri = subitem[1]


def process_reference(elm, registry, container):
    reference = Reference()
    reference.text = elm.text
    container.add_content(reference)
    for subitem in elm.items():
        if subitem[0] == 'name':
            reference.name = subitem[1]
        elif subitem[0] == 'uri' or subitem[0] == 'refuri':
            reference.uri = subitem[1]


def process_raw(elm, registry, container):
    raw = Raw()
    container.add_content(raw)
    for subitem in elm.items():
        if subitem[0] == 'format':
            raw.format = subitem[1]
    raw.text = elm.text


def process_image(elm, registry, container):
    img = Image()
    container.add_content(img)
    for subitem in elm.items():
        if subitem[0] == 'alt':
            img.alt = subitem[1]
        elif subitem[0] == 'width':
            img.width = subitem[1]
        elif subitem[0] == 'height':
            img.height = subitem[1]
        elif subitem[0] == 'uri':
            img.uri = subitem[1]


def process_block_quote(elm, registry, container):
    block_quote = BlockQuote()
    container.add_content(block_quote)
    for _elm in list(elm):
        registry.process(_elm, block_quote)


def process_literal_block(elm, registry, container):
    literal_block = LiteralBlock()
    container.add_content(literal_block)
    literal_block.text = elm.text


def process_figure(elm, registry, container):
    figure = Figure()
    container.add_content(figure)
    for _elm in list(elm):
        registry.process(_elm, figure)


def process_topic(elm, registry, container):
    topic = Topic()
    container.add_content(topic)
    for _elm in list(elm):
        registry.process(_elm, topic)


def process_substitution_definition(elm, registry, container):
    substitution_definition = SubstitutionDefinition()
    container.add_content(substitution_definition)
    for _item in elm.items():
        if _item[0] == 'names':
            title = Title()
            title.text = _item[1]
            substitution_definition.title = title
    for _elm in list(elm):
        registry.process(_elm, substitution_definition)


def process_caption(elm, registry, container):
    caption = Caption()
    caption.text = elm.text
    container.add_content(caption)


def process_note(elm, registry, container):
    note = Note()
    container.add_content(note)
    for _elm in list(elm):
        registry.process(_elm, note)


def process_title(elm, registry, container):
    title = Title()
    title.text = elm.text
    container.title = title


def process_table(elm, registry, container):
    table = Table()
    container.add_content(table)
    for _elm in list(elm):
        registry.process(_elm, table)


def process_tgroup(elm, registry, container):
    tgroup = Tgroup()
    container.add_content(tgroup)
    for _elm in list(elm):
        registry.process(_elm, container)


def process_colspec(elm, registry, container):
    colspec = ColSpec()
    container.add_content(colspec)
    for item in elm.items():
        if hasattr(colspec, item[0]):
            setattr(colspec, item[1])


def process_thead(elm, registry, container):
    thead = Thead()
    container.add_content(thead)
    for _elm in list(elm):
        registry.process(_elm, thead)


def process_row(elm, registry, container):
    row = Row()
    if container.__class__ == Thead:
        row.header = True
    container.add_content(row)
    for _elm in list(elm):
        registry.process(_elm, row)


def process_entry(elm, registry, container):
    entry = Entry()
    container.add_content(entry)
    for _elm in list(elm):
        registry.process(_elm, entry)


def process_tbody(elm, registry, container):
    tbody = Tbody()
    container.add_content(tbody)
    for _elm in list(elm):
        registry.process(_elm, tbody)


# TODO test
def process_enumerated_list(elm, registry, container):
    enumerated_list = EnumeratedList()
    container.add_content(enumerated_list)
    for _elm in list(elm):
        registry.process(_elm, enumerated_list)


# TODO test
def process_transition(elm, registry, container):
    container.add_content(Transition())


def process_document(elm, registry, container):
    for _elm in list(elm):
        registry.process(_elm, container)


def process_paragraph(elm, registry, container):
    paragraph = Paragraph()
    paragraph.text = ''.join(elm.itertext())
    container.add_content(paragraph)
    for _elm in list(elm):
        registry.process(_elm, paragraph)


def process_strong(elm, registry, container):
    strong = Strong()
    strong.text = elm.text
    container.add_content(strong)


def process_emphasis(elm, registry, container):
    emphasis = Emphasis()
    emphasis.text = elm.text
    container.add_content(emphasis)


def process_section(elm, registry, container, depth=0):
    section = Section()
    container.add_content(section)
    for _elm in list(elm):
        registry.process(_elm, section)


# TODO test
def _process_comment(elm, registry, story):
    _field_list = list(elm)[0]
    commentator = ''
    url = ''
    body = []
    create_date = None
    for _field in _field_list:
        if 'commentator' == _field[0].text:
            commentator = _field[1][0].text
        elif 'url' == _field[0].text:
            url = _field[1].text
        elif 'body' == _field[0].text:
            body += [x.text for x in _field[1]]
        elif 'create_date' == _field[0].text:
            create_date = datetime.strptime(
                _field[1][0].text, '%Y-%m-%d %H:%M'
            )
        else:
            logger.warning(_field[0].text)
    c = Comment()
    c.commentator = commentator
    c.url = url
    for b in body:
        c.add_content(b)
    c.create_date = create_date
    story.comments.append(c)


def process_docinfo(elm, registry, story):
    for _elm in list(elm):
        if len(_elm) == 2:
            if 'field_name' == _elm[0].tag:
                field_name = process_field_name(_elm[0], registry, story)
                if field_name == 'slug':
                    story.slug = process_field_body(
                        _elm[1], registry, story)[0]
                elif field_name == 'author':
                    story.author = process_field_body(
                        _elm[1], registry, story)[0]
                elif field_name == 'date':
                    story.date = datetime.strptime(
                        _elm.text, '%Y-%m-%d %H:%M'
                    )
                elif field_name == 'comment':
                    _process_comment(_elm[1], registry, story)
            else:
                logger.warning(
                    "elm.tag '{0}' doesn't process in process_docinfo.".format(
                        _elm[0].tag))
        elif 'date' == _elm.tag:
            story.date = datetime.strptime(
                _elm.text, '%Y-%m-%d %H:%M'
            )
        elif 'author' == _elm.tag:
            story.author = _elm.text


class FunctionRegistry(dict):
    def __init__(self, *args, **kwargs):
        for key, value in kwargs.items():
            if hasattr(value, '__call__'):
                self[key] = value
            else:
                raise ValueError('accept only callable.')

    def __setattr__(self, key, value):
        if hasattr(value, '__call__'):
            self[key] = value
            logger.debug(
                'register process function: {0}'.format(
                    self[key]))
        else:
            raise ValueError('accept only callable.')

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            object.__getattribute__(self, key)

    def register(self, name, func):
        setattr(self, name, func)

    def process(self, elm, container):
        _processor_name = 'process_{0}'.format(elm.tag)
        if hasattr(self, _processor_name):
            logger.debug('---------------')
            logger.debug(getattr(self, _processor_name).__name__)
            logger.debug(getattr(self, _processor_name).__code__.co_varnames)
            _fnc = getattr(self, _processor_name)
            return _fnc(elm, self, container)
        else:
            logger.warning(
                'processor {0} is not defined and element ignored.'.format(
                    _processor_name))
